require("player/Player")
require("player/SpriteAnimation")
require("camera")
require("player/coin")
require("bullet")

function love.load()
    width = love.graphics.getWidth()
    height = love.graphics.getHeight()
    g = love.graphics
    love.graphics.setBackgroundColor(0, 198, 198)
    bullet = g.newImage("bullet.png")

    -- Load up the map
    loader = require("AdvTiledLoader.Loader")
    loader.path = "maps/"
    map = loader.load("map01.tmx")
    map:setDrawRange(0, 0, map.width * map.tileWidth, map.height * map.tileHeight)

    -- restrict the camera
    camera:setBounds(0, 0, map.width * map.tileWidth - width, map.height * map.tileHeight - height)

    p = Player:new()

    p.x = 300
    p.y = 10
    p.width = 32
    p.height = 32
    p.jumpSpeed = -800
    p.runSpeed = 500

    gravity = 1800
    hasJumped = false
    delay = 120

    -- Load player animation
    animation = SpriteAnimation:new("player/robosprites.png", 32, 32, 2, 1)
    animation:load(delay)

    -- create coin animation
    coinSprites = SpriteAnimation:new("player/coin.png", 32, 32, 2, 1)
    coinSprites:load(delay)

    score = 0

    -- Place random coins around the map
    math.randomseed(os.time())
    numCoins = 20
    coins = {}
    for i = 1, numCoins do
        local coinCollides = true
        while coinCollides do -- try to place a coin on a random spot around the map
            local coinX = math.random(1, map.width - 1) * map.tileWidth + map.tileWidth / 2
            local coinY = math.random(1, map.height - 1) * map.tileHeight + map.tileHeight / 2
            coins[i] = coin:new(coinX, coinY)

            -- if tile is occupied, try again
            coinCollides = coins[i]:isColliding(map)
        end
    end
end

function love.update(dt)
    -- check controls
    if love.keyboard.isDown("d") then
        p:moveRight()
        animation:flip(false, false)
        p.facingRight = true
        p.facingLeft = false
    end
    if love.keyboard.isDown("a") then
        p:moveLeft()
        animation:flip(true, false)
        p.facingRight = false
        p.facingLeft = true
    end
    if love.keyboard.isDown("w") and not(hasJumped) then
        hasJumped = true
        p:jump()
    end

    -- update the player's position and check for collisions
    p:update(dt, gravity, map)

    -- update the sprite animation
    if (p.state == "stand") then
        animation:switch(1, 2, 200)
    end
    if (p.state == "moveRight") or (p.state == "moveLeft") then
        animation:switch(1, 2, 120)
    end
    if (p.state == "jump") or (p.state == "fall") then
        animation:reset()
        animation:switch(1, 2, 300)
    end

    animation:update(dt)

    -- update coin animations and check for player collisions
    for i in ipairs(coins) do
        coins[i]:update(dt)

        -- if player collides, add to score and remove coin
        if coins[i]:touchesObject(p) then
            score = score + 1
            table.remove(coins, i)
        end
    end

    -- center the camera on the player
    camera:setPosition(math.floor(p.x - width / 2), math.floor(p.y - height / 2))
end

function love.draw()
    camera:set()

    -- round our x, y values
    local x, y = math.floor(p.x), math.floor(p.y)

    -- draw the map
    map:draw()

    -- draw coins
    for i in ipairs(coins) do
        coinSprites:start(coins[i].frame)
        coinSprites:draw(coins[i].x - coins[i].width / 2, coins[i].y - coins[i].height / 2)
    end

    -- draw the player
    animation:draw(x - p.width / 2, y - p.height / 2)

    camera:unset()

    -- debug information
    local tileX = math.floor(p.x / map.tileWidth)
    local tileY = math.floor(p.y / map.tileHeight)

    love.graphics.setColor(255, 255, 255)
    love.graphics.print("Player coordinates: ("..x..","..y..")", 5, 5)
    love.graphics.print("Current state: "..p.state, 5, 20)
    love.graphics.print("Current tile: ("..tileX..", "..tileY..")", 5, 35)
    -- draw the score
    love.graphics.print("Score: "..score, 700, 5)
end

function love.keyreleased(key)
    if key == "escape" then
        love.event.push("quit")   -- actually causes the app to quit
    end
    if (key == "d") or (key == "a") then
        p:stop()
    end
    if (key == "w") then
        hasJumped = false
    end
end

function math.clamp(x, min, max)
  return x < min and min or (x > max and max or x)
end
